unit Tcl;
{* Basic Interface into the TCL Interpretor.  This is about 1/1000th of *}
{* What's actually in tcl.h, but we're getting there!                   *}

interface

type

	{* Arguments to a procedure.  We define it in a type like this for ease *}
	{* Of function declaraction.                                            *}
	TArgs = array of PChar;
	PArgs = ^TArgs;

	{* Client data Type *}
	PClientData = ^TClientData;
	TClientData = Integer;

	{* This is used by about half the function types, shouldn't this already *}
	{* exists somewhere?                                                     *}
	PInteger = ^Integer;

	{* TCL Value Types *}
	TTclValueType = (TCL_INT, TCL_DOUBLE, TCL_EITHER);
	PTclValue = ^TTclValue;
	Tcl_Value = packed record
		valType: TTclValueType;
		intValue: LongInt;
		doubleValue: Double;
	end;
	TTclValue = Tcl_Value;

	{* Advanced Declaration of a Tcl_Interp and Tcl_Obj, as needed for the *}
	{* parameters to these functions.  (We need the functions, in turn, to *}
	{* flesh out the record types, so this is neccessary.                  *}
	PTclInterp = ^TTclInterp;
	PTclObj = ^TTclObj;

	{* An array of pointers to the object type, used in Tcl_ObjCmdProc *}
	{* And a few of the Tcl_NewblahObj functions.                      *}
	TObjArgs = array of PTclObj;
	PObjArgs = ^TObjArgs;

	{* These types are somehow extracted from the .DLL, but fuck if I can *}
	{* figure out how to do it in Delphi!  We just call them pointers for *}
	{* now, but the real translation is shown in the comment for a TclVar *}
	PTclAsyncHandler	= Pointer;
	PTclChannel			= Pointer;
	PTclCommand			= Pointer;
	PTclCondition		= Pointer;
	PTclEncodingState	= Pointer;
	PTclEncoding		= Pointer;
	PTclEvent			= Pointer;
	PTclMutex			= Pointer;
	PTclPid				= Pointer;
	PTclRegExp			= Pointer;
	PTclThreadDataKey	= Pointer;
	PTclThreadId		= Pointer;
	PTclTimerToken 		= Pointer;
	PTclTrace			= Pointer;
	PTclVar				= Pointer;
	{*
	TTclVar = Tcl_Var_; extern; ???
	PTclVar = ^TTclVar;
	*}

	{* Various procedure types TCL uses. *}
	TTclAppInitProc = function(interp: PTclInterp): Integer; cdecl;
	TTclAsyncProc = function(clientData: PClientData; interp: PTclInterp;
								code: Integer): Integer; cdecl;
	TTclChannelProc = procedure(clientData: PClientData; mask: Integer); cdecl;
	TTclCloseProc = procedure(clientData: PClientData); cdecl;
	TTclCmdDeleteProc = procedure(clientData: PClientData); cdecl;
	TTclCmdProc = function(clientData: PClientData; interp: PTclInterp;
								argc: Integer; argv: PArgs): Integer; cdecl;
	TTclCmdTraceProc = procedure(clientData: PClientData; interp: PTclInterp;
								level: Integer; command: PChar;
								proc: TTclCmdProc; cmdClientDatA: PClientData;
								argc: Integer; const argv: PArgs); cdecl;
	TTclDupInternalRepProc = procedure(srcPtr, dupPtr: PTclObj); cdecl;
	TTclEncodingConvertProc = function(clientData: PClientData; src: PChar;
								srcLen, flags: Integer;
								statePtr: PTclEncodingState; dst: PChar;
								dstLen: Integer; scrReadPtr: PInteger;
								dstWrotePtr, dstCharsPtr: PInteger):
								Integer; cdecl;
	TTclEventProc = function(evPtr: PTclEvent; flags: Integer): Integer; cdecl;
	TTclEventCheckProc = procedure(clientData: PClientData; flags: Integer);
								cdecl;
	TTclEventDeleteProc = function(evPtr: PTclEvent; clientData: PClientData):
								Integer; cdecl;
	TTclEventSetupProc = procedure(clientData: PClientData; flags: Integer);
								cdecl;
	TTclExitProc = procedure(clientData: PClientData); cdecl;
	TTclFileProc = procedure(clientData: PClientData; mask: Integer); cdecl;
	TTclFileFreeProc = procedure(clientData: PClientData); cdecl;
	TTclFreeInternalRepProc = procedure(objPtr: PTclObj); cdecl;
	TTclFreeProc = procedure(blockPtr: PChar); cdecl;
	TTclIdleProc = procedure(clientData: PClientData); cdecl;
	TTclInterpDeleteProc = procedure(clientData: PClientData;
								interp: PTclInterp); cdecl;
	TTclMathProc = function(clientData: PClientData; interp: PTclInterp;
								args, resultPtr: PTclValue): Integer; cdecl;
	TTclNamespaceDeletEProc = procedure(clientData: PClientData); cdecl;
	TTclObjCmdProc = function(clientData: PClientData; interp: PTclInterp;
								objc: Integer; const objv: PObjArgs): Integer;
	TTclPackageInitProc = function(interp: PTclInterp): Integer;
	TTclTcpAcceptProc = procedure(callbackData: PClientData;
								chan: PTclChannel; address: PChar;
								port: Integer);
	TTclTimerProc = procedure(clientData: PClientData);
	TTclSetFromAnyProc = function(interp: PTclInterp; objPtr: PTclObj): Integer;
	TTclUpdateStringProc = procedure(objPtr: PTclObj);
	TTclVarTraceProc = function(clientData: PClientData; interp: PTclInterp;
								part1, part2: PChar; flags: Integer): PChar;

	(*
	 * The following structure represents a type of object, which is a
	 * particular internal representation for an object plus a set of
	 * procedures that provide standard operations on objects of that type.
	 *)
	PTclObjType = ^TTclObjType;
	Tcl_ObjType = packed record
		name			: PChar;
		freeIntRepProc	: TTclFreeInternalRepProc;
		dupIntRepProc	: TTclDupInternalRepProc;
		updateStringProc: TTclUpdateStringProc;
		setFromAnyProc	: TTclSetFromAnyProc;
	end;
	TTclObjType = Tcl_ObjType;

	(*
	 * One of the following structures exists for each object in the Tcl
	 * system. An object stores a value as either a string, some internal
	 * representation, or both.
	 *)
	 Tcl_Obj = packed record
		refCount	: Integer;
		bytes		: PChar;	(* This points to the first byte of the
								 * object's string representation. The array
								 * must be followed by a null byte (i.e., at
								 * offset length) but may also contain
								 * embedded null characters. The array's
								 * storage is allocated by ckalloc. NULL
								 * means the string rep is invalid and must
								 * be regenerated from the internal rep.
								 * Clients should use Tcl_GetStringFromObj
								 * or Tcl_GetString to get a pointer to the
								 * byte array as a readonly value. *)
		length		: Integer;
		typePtr		: PTclObjType;	(* Denotes the object's type. Always
									 * corresponds to the type of the object's
									 * internal rep. NULL indicates the object
									 * has no internal rep (has no type). *)
	case Integer of  {* internalRep *}
		0: (longValue		: LongInt);
		1: (doubleValue		: Double);
		2: (otherValuePtr	: Pointer);
		3: (twoPtrValue		: packed record
				ptr1, ptr2: Pointer;
			end);
	end;
	TTclObj = Tcl_Obj;

	(*
	 * Data structures defined opaquely in this module. The definitions below
	 * just provide dummy types. A few fields are made visible in Tcl_Interp
	 * structures, namely those used for returning a string result from
	 * commands. Direct access to the result field is discouraged in Tcl 8.0.
	 * The interpreter result is either an object or a string, and the two
	 * values are kept consistent unless some C code sets interp->result
	 * directly. Programmers should use either the procedure Tcl_GetObjResult()
	 * or Tcl_GetStringResult() to read the interpreter's result. See the
	 * SetResult man page for details.
	 *
	 * Note: any change to the Tcl_Interp definition below must be mirrored
	 * in the "real" definition in tclInt.h.
	 *
	 * Note: Tcl_ObjCmdProc procedures do not directly set result and freeProc.
	 * Instead, they set a Tcl_Obj member in the "real" structure that can be
	 * accessed with Tcl_GetObjResult() and Tcl_SetObjResult().
	 *)
	Tcl_Interp = packed record
		Result    : PChar;          (* If the last command returned a string
									 * result, this points to it. *)
		freeProc  : TTclFreeProc;	(* Zero means the string result is
									 * statically allocated. TCL_DYNAMIC means
									 * it was allocated with ckalloc and should
									 * be freed with ckfree. Other values give
									 * the address of procedure to invoke to
									 * free the result. Tcl_Eval must free it
									 * before executing next command. *)
		erorrLine : Integer;		(* When TCL_ERROR is returned, this gives
									 * the line number within the command where
									 * the error occurred (1 if first line). *)
	end;
	TTclInterp = Tcl_Interp;

	(*
	 * The following definitions support Tcl's namespace facility.
	 * Note: the first five fields must match exactly the fields in a
	 * Namespace structure (see tcl.h).
	 *)
	PTclnameSpace = ^TTclNamespace;
	Tcl_Namespace = packed record
		name		: PChar;	(* The namespace's name within its parent
								 * namespace. This contains no ::'s. The
								 * name of the global namespace is ""
								 * although "::" is an synonym.
								 *)
		fullName    : PChar;	(* The namespace's fully qualified name.
								 * This starts with ::. *)
		clientData	: PClientData;
		deleteProc	: TTclNamespaceDeleteProc;
		parentPtr	: PTclNamespace;
	end;
	TTclNamespace = Tcl_NameSpace;

	(*
	 * Information about commands that is returned by Tcl_GetCommandInfo and
	 * passed to Tcl_SetCommandInfo. objProc is an objc/objv object-based
	 * command procedure while proc is a traditional Tcl argc/argv
	 * string-based procedure. Tcl_CreateObjCommand and Tcl_CreateCommand
	 * ensure that both objProc and proc are non-NULL and can be called to
	 * execute the command. However, it may be faster to call one instead of
	 * the other. The member isNativeObjectProc is set to 1 if an
	 * object-based procedure was registered by Tcl_CreateObjCommand, and to
	 * 0 if a string-based procedure was registered by Tcl_CreateCommand.
	 * The other procedure is typically set to a compatibility wrapper that
	 * does string-to-object or object-to-string argument conversions then
	 * calls the other procedure.
	 *)
	 PTclCmdInfo = ^TTclCmdInfo;
	 Tcl_CmdInfo = packed record
		isNativeObjectProc	: Integer;
		objProc				: TTclObjCmdProc;
		objClientData		: PClientData;
		proc				: TTclCmdProc;
		clientData			: PClientData;
		deleteProc			: TTclCmdDeleteProc;
		deleteData			: PClientData;
		namespacePtr		: PTclNameSpace;
	 end;
	 TTclCmdInfo = Tcl_CmdInfo;

const
	{* Required Version identifiers. Hopefully we'll be able to ignore *}
	{* These and use any version of TCL.DLL                            *}
	TCL_MAJOR_VERSION   = 8;
	TCL_MINOR_VERSION   = 1;
	TCL_RELEASE_LEVEL   = 1;
	TCL_RELEASE_SERIAL  = 1;

	TCL_VERSION	    	= '8.1';
	TCL_PATCH_LEVEL	    = '8.1b1';

	(*
	 * When a TCL command returns, the interpreter contains a result from the
	 * command. Programmers are strongly encouraged to use one of the
	 * procedures Tcl_GetObjResult() or Tcl_GetStringResult() to read the
	 * interpreter's result. See the SetResult man page for details. Besides
	 * this result, the command procedure returns an integer code, which is
	 * one of the following:
	 *
	 * TCL_OK			Command completed normally; the interpreter's
	 *					result contains	the command's result.
	 * TCL_ERROR		The command couldn't be completed successfully;
	 *					the interpreter's result describes what went wrong.
	 * TCL_RETURN		The command requests that the current procedure
	 *					return; the interpreter's result contains the
	 *					procedure's return value.
	 * TCL_BREAK		The ccmmand requests that the innermost loop
	 *					be exited; the interpreter's result is meaningless.
	 * TCL_CONTINUE		Go on to the next iteration of the current loop;
	 *					the interpreter's result is meaningless.
	 *)
	TCL_OK			= 0;
	TCL_ERROR		= 1;
	TCL_RETURN		= 2;
	TCL_BREAK		= 3;
	TCL_CONTINUE	= 4;

	TCL_RESULT_SIZE = 200;

(*
 * Macros and definitions that help to debug the use of Tcl objects.
 * When TCL_MEM_DEBUG is defined, the Tcl_New* declarations are
 * overridden to call debugging versions of the object creation procedures.
 *)
function TclNewBooleanObj(boolValue: Integer): PTclObj; cdecl;
function TclNewByteArrayObj(bytes: PChar; length: Integer): PTclObj; cdecl;
function TclNewDoubleObj(doubleValue: Double): PTclObj; cdecl;
function TclNewIntObj(intValue: Integer): PTclObj; cdecl;
function TclNewListObj(objc: Integer; objv: PObjArgs): PTclObj; cdecl;
function TclNewLongObj(longValue: LongInt): PTclObj; cdecl;
function TclNewObj: PTclObj; cdecl;
function TclNewStringObj(bytes: PChar; length: Integer): PTclObj; cdecl;

procedure TclIncrRefCount(objPtr: PTclObj); cdecl;
procedure TclDecrRefCount(objPtr: PTclOBj); cdecl;
function TclIsShared(objPtr: PTclObj): Integer; cdecl;

(*
 * Exported Tcl procedures:
 *)


procedure TclAddErrorInfo(interp: PTclInterp; const msg: PChar); cdecl;
procedure TclAddObjErrorInfo(interp: PTclInterp; const msg: PChar); cdecl;
procedure TclAllowExceptions(interp: PTclInterp); cdecl;
function TclAppendAllObjTypes(interp: PTclInterp; objPtr: PTclObj): Integer;
								cdecl;
procedure TclAppendElement(interp: PTclInterp; element: PChar); cdecl;
procedure TclAppendResult(interp: PTclInterp; res,dummy: PChar); cdecl;
procedure TclAppendObjToObj(objPtr, appendObjPtr: PTclObj); cdecl;
procedure TclAppendToObj(objPtr: PTclOBj; bytes: PChar; length: Integer); cdecl;
(*
EXTERN void		Tcl_AppendStringsToObj _ANSI_ARGS_(
				TCL_VARARGS(Tcl_Obj *,interp));
*)
function TclAppInit(interp: PTclInterp): Integer; cdecl;
procedure TclAlertNotifier(clientData: PClientData); cdecl;
function TclAsyncCreate(proc: TTclAsyncProc; clientData: PClientData):
								PTclAsyncHandler; cdecl;
procedure TclAsyncDelete(async: PTclAsyncHandler); cdecl;
function TclAsyncInvoke(interp: PTclInterp; code: Integer): Integer; cdecl;
procedure TclAsnycMark(async: PTclAsyncHandler); cdecl;
function TclAsyncReady: Integer; cdecl;
procedure TclBackgroundError(interp: PTclInterp); cdecl
function TclBackslash(const src: PChar; readPtr: PInteger): PChar; cdecl;
function TclBadChannelOption(interp: PTclInterp; optionName,
								optionList: PChar): Integer; cdecl;
procedure TclCallWhenDelete(interp: PTclInterp; proc: TTclInterpDeleteProc;
								clientData: PClientData); cdecl;
procedure TclCancelIdleCall(idleProc: TTclIdleProc; clientData: PClientData);
								cdecl;
(*
#define Tcl_Ckalloc Tcl_Alloc
#define Tcl_Ckfree Tcl_Free
#define Tcl_Ckrealloc Tcl_Realloc
EXTERN int		Tcl_Close _ANSI_ARGS_((Tcl_Interp *interp,
					Tcl_Channel chan));
EXTERN int		Tcl_CommandComplete _ANSI_ARGS_((char *cmd));
EXTERN char *		Tcl_Concat _ANSI_ARGS_((int argc, char **argv));
EXTERN Tcl_Obj *	Tcl_ConcatObj _ANSI_ARGS_((int objc,
			    Tcl_Obj *CONST objv[]));
EXTERN int		Tcl_ConvertCountedElement _ANSI_ARGS_((CONST char *src,
				int length, char *dst, int flags));
EXTERN int		Tcl_ConvertElement _ANSI_ARGS_((CONST char *src,
			    char *dst, int flags));
EXTERN int		Tcl_ConvertToType _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *objPtr, Tcl_ObjType *typePtr));
EXTERN int		Tcl_CreateAlias _ANSI_ARGS_((Tcl_Interp *slave,
			    char *slaveCmd, Tcl_Interp *target,
					char *targetCmd, int argc, char **argv));
EXTERN int		Tcl_CreateAliasObj _ANSI_ARGS_((Tcl_Interp *slave,
			    char *slaveCmd, Tcl_Interp *target,
        		    char *targetCmd, int objc,
					Tcl_Obj *CONST objv[]));
EXTERN Tcl_Channel	Tcl_CreateChannel _ANSI_ARGS_((
    			    Tcl_ChannelType *typePtr, char *chanName,
                            ClientData instanceData, int mask));
EXTERN void		Tcl_CreateChannelHandler _ANSI_ARGS_((
				Tcl_Channel chan, int mask,
							 Tcl_ChannelProc *proc, ClientData clientData));
EXTERN void		Tcl_CreateCloseHandler _ANSI_ARGS_((
				Tcl_Channel chan, Tcl_CloseProc *proc,
							 ClientData clientData));
*)
function TclCreateCommand(interp: PTclInterp; cmdName: PChar;
								TclCmdProc: TTclCmdProc; clientData: PClientData;
								TclCmdDeleteProc: TTclCmdDeleteProc):
								PTclCommand; cdecl;
(*
EXTERN void		Tcl_CreateEventSource _ANSI_ARGS_((
				Tcl_EventSetupProc *setupProc,
				Tcl_EventCheckProc *checkProc,
				ClientData clientData));
EXTERN Tcl_Encoding	Tcl_CreateEncoding _ANSI_ARGS_((
				Tcl_EncodingType *typePtr));
*)
procedure TclCreateExitHandler(TclCmdProc: TTclCmdProc;
								clientData: PClientData); cdecl;
procedure TclCreateFileHandler(fd, mask: Integer; proc: TTclFileProc;
								clientData: PClientData); cdecl;
function TclCreateInterp: PTclInterp; cdecl;

(*
EXTERN void		Tcl_CreateMathFunc _ANSI_ARGS_((Tcl_Interp *interp,
				char *name, int numArgs, Tcl_ValueType *argTypes,
				Tcl_MathProc *proc, ClientData clientData));
EXTERN Tcl_Command	Tcl_CreateObjCommand _ANSI_ARGS_((
				Tcl_Interp *interp, char *cmdName,
				Tcl_ObjCmdProc *proc, ClientData clientData,
				Tcl_CmdDeleteProc *deleteProc));
EXTERN Tcl_Interp *	Tcl_CreateSlave _ANSI_ARGS_((Tcl_Interp *interp,
					char *slaveName, int isSafe));
EXTERN void		Tcl_CreateThreadExitHandler
				_ANSI_ARGS_((Tcl_ExitProc *proc,
				ClientData clientData));
EXTERN Tcl_TimerToken	Tcl_CreateTimerHandler _ANSI_ARGS_((int milliseconds,
				Tcl_TimerProc *proc, ClientData clientData));
EXTERN Tcl_Trace	Tcl_CreateTrace _ANSI_ARGS_((Tcl_Interp *interp,
				int level, Tcl_CmdTraceProc *proc,
				ClientData clientData));
EXTERN char *		Tcl_DbCkalloc _ANSI_ARGS_((unsigned int size,
			    char *file, int line));
EXTERN int		Tcl_DbCkfree _ANSI_ARGS_((char *ptr,
			    char *file, int line));
EXTERN char *		Tcl_DbCkrealloc _ANSI_ARGS_((char *ptr,
			    unsigned int size, char *file, int line));
EXTERN void		Tcl_DbDecrRefCount _ANSI_ARGS_((Tcl_Obj *objPtr,
			    char *file, int line));
EXTERN void		Tcl_DbIncrRefCount _ANSI_ARGS_((Tcl_Obj *objPtr,
				char *file, int line));
EXTERN int		Tcl_DbIsShared _ANSI_ARGS_((Tcl_Obj *objPtr,
				char *file, int line));
EXTERN Tcl_Obj *	Tcl_DbNewBooleanObj _ANSI_ARGS_((int boolValue,
                            char *file, int line));
EXTERN Tcl_Obj *	Tcl_DbNewDoubleObj _ANSI_ARGS_((double doubleValue,
							 char *file, int line));
EXTERN Tcl_Obj *	Tcl_DbNewListObj _ANSI_ARGS_((int objc,
			    Tcl_Obj *CONST objv[], char *file, int line));
EXTERN Tcl_Obj *	Tcl_DbNewLongObj _ANSI_ARGS_((long longValue,
                            char *file, int line));
EXTERN Tcl_Obj *	Tcl_DbNewObj _ANSI_ARGS_((char *file, int line));
EXTERN Tcl_Obj *	Tcl_DbNewStringObj _ANSI_ARGS_((CONST char *bytes,
			    int length, char *file, int line));
EXTERN void		Tcl_DeleteAssocData _ANSI_ARGS_((Tcl_Interp *interp,
                            char *name));
*)
function TclDeleteCommand(interp: PTclInterp; cmdName: PChar): Integer; cdecl;
(*
EXTERN int		Tcl_DeleteCommandFromToken _ANSI_ARGS_((
				Tcl_Interp *interp, Tcl_Command command));
EXTERN void		Tcl_DeleteChannelHandler _ANSI_ARGS_((
    			    Tcl_Channel chan, Tcl_ChannelProc *proc,
                            ClientData clientData));
EXTERN void		Tcl_DeleteCloseHandler _ANSI_ARGS_((
			    Tcl_Channel chan, Tcl_CloseProc *proc,
                            ClientData clientData));
EXTERN void		Tcl_DeleteEvents _ANSI_ARGS_((
			    Tcl_EventDeleteProc *proc,
							 ClientData clientData));
EXTERN void		Tcl_DeleteEventSource _ANSI_ARGS_((
				Tcl_EventSetupProc *setupProc,
				Tcl_EventCheckProc *checkProc,
			    ClientData clientData));
EXTERN void		Tcl_DeleteExitHandler _ANSI_ARGS_((Tcl_ExitProc *proc,
				ClientData clientData));
EXTERN void		Tcl_DeleteFileHandler _ANSI_ARGS_((int fd));
EXTERN void		Tcl_DeleteHashEntry _ANSI_ARGS_((
			    Tcl_HashEntry *entryPtr));
EXTERN void		Tcl_DeleteHashTable _ANSI_ARGS_((
			    Tcl_HashTable *tablePtr));
*)
procedure TclDeleteInterp(interp: PTclInterp); cdecl;
(*
EXTERN void		Tcl_DeleteThreadExitHandler
			    _ANSI_ARGS_((Tcl_ExitProc *proc,
			    ClientData clientData));
EXTERN void		Tcl_DeleteTimerHandler _ANSI_ARGS_((
			    Tcl_TimerToken token));
EXTERN void		Tcl_DeleteTrace _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Trace trace));
EXTERN void		Tcl_DetachPids _ANSI_ARGS_((int numPids, Tcl_Pid *pidPtr));
EXTERN void		Tcl_DiscardResult _ANSI_ARGS_((
			    Tcl_SavedResult *statePtr));
EXTERN void		Tcl_DontCallWhenDeleted _ANSI_ARGS_((
			    Tcl_Interp *interp, Tcl_InterpDeleteProc *proc,
			    ClientData clientData));
EXTERN int		Tcl_DoOneEvent _ANSI_ARGS_((int flags));
EXTERN void		Tcl_DoWhenIdle _ANSI_ARGS_((Tcl_IdleProc *proc,
			    ClientData clientData));
EXTERN char *		Tcl_DStringAppend _ANSI_ARGS_((Tcl_DString *dsPtr,
				CONST char *string, int length));
EXTERN char *		Tcl_DStringAppendElement _ANSI_ARGS_((
				Tcl_DString *dsPtr, CONST char *string));
EXTERN void		Tcl_DStringEndSublist _ANSI_ARGS_((Tcl_DString *dsPtr));
EXTERN void		Tcl_DStringFree _ANSI_ARGS_((Tcl_DString *dsPtr));
EXTERN void		Tcl_DStringGetResult _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_DString *dsPtr));
EXTERN void		Tcl_DStringInit _ANSI_ARGS_((Tcl_DString *dsPtr));
EXTERN void		Tcl_DStringResult _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_DString *dsPtr));
EXTERN void		Tcl_DStringSetLength _ANSI_ARGS_((Tcl_DString *dsPtr,
				int length));
EXTERN void		Tcl_DStringStartSublist _ANSI_ARGS_((
				Tcl_DString *dsPtr));
EXTERN Tcl_Obj *	Tcl_DuplicateObj _ANSI_ARGS_((Tcl_Obj *objPtr));
EXTERN int		Tcl_Eof _ANSI_ARGS_((Tcl_Channel chan));
EXTERN char *		Tcl_ErrnoId _ANSI_ARGS_((void));
EXTERN char *		Tcl_ErrnoMsg _ANSI_ARGS_((int err));
*)
function TclEval(interp: PTclInterp; command: PChar): Integer; cdecl;
function TclEval2(interp: PTclInterp; script: PChar; numBytes, flags: Integer):
								Integer; cdecl;
function TclEvalFile(interp: PTclInterp; fileName: PChar): Integer; cdecl;
(*
EXTERN int		Tcl_EvalObjv _ANSI_ARGS_ ((Tcl_Interp *interp,
			    int objc, Tcl_Obj *CONST objv[], int flags));
EXTERN int		Tcl_EvalObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *objPtr, int flags));
EXTERN void		Tcl_EventuallyFree _ANSI_ARGS_((ClientData clientData,
			    Tcl_FreeProc *freeProc));
EXTERN void		Tcl_Exit _ANSI_ARGS_((int status));
EXTERN void		Tcl_ExitThread _ANSI_ARGS_((int status));
EXTERN int		Tcl_ExposeCommand _ANSI_ARGS_((Tcl_Interp *interp,
        		    char *hiddenCmdToken, char *cmdName));
EXTERN int		Tcl_ExprBoolean _ANSI_ARGS_((Tcl_Interp *interp,
				char *string, int *ptr));
EXTERN int		Tcl_ExprBooleanObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *objPtr, int *ptr));
EXTERN int		Tcl_ExprDouble _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, double *ptr));
EXTERN int		Tcl_ExprDoubleObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *objPtr, double *ptr));
EXTERN int		Tcl_ExprLong _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, long *ptr));
EXTERN int		Tcl_ExprLongObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *objPtr, long *ptr));
EXTERN int		Tcl_ExprObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *objPtr, Tcl_Obj **resultPtrPtr));
EXTERN int		Tcl_ExprString _ANSI_ARGS_((Tcl_Interp *interp,
				char *string));
EXTERN int		Tcl_ExternalToUtf _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Encoding encoding, CONST char *src, int srcLen,
				int flags, Tcl_EncodingState *statePtr, char *dst,
				int dstLen, int *srcReadPtr, int *dstWrotePtr,
				int *dstCharsPtr));
EXTERN char *		Tcl_ExternalToUtfDString _ANSI_ARGS_((
			    Tcl_Encoding encoding, CONST char *src,
			    int srcLen, Tcl_DString *dsPtr));
EXTERN void		Tcl_Finalize _ANSI_ARGS_((void));
EXTERN void		Tcl_FinalizeThread _ANSI_ARGS_((void));
EXTERN void		Tcl_FinalizeNotifier _ANSI_ARGS_((
			    ClientData clientData));
EXTERN void		Tcl_FindExecutable _ANSI_ARGS_((CONST char *argv0));
EXTERN Tcl_HashEntry *	Tcl_FirstHashEntry _ANSI_ARGS_((
				Tcl_HashTable *tablePtr,
			    Tcl_HashSearch *searchPtr));
EXTERN int		Tcl_Flush _ANSI_ARGS_((Tcl_Channel chan));
EXTERN void		Tcl_FreeEncoding _ANSI_ARGS_((Tcl_Encoding encoding));
EXTERN void		TclFreeObj _ANSI_ARGS_((Tcl_Obj *objPtr));
EXTERN void		Tcl_FreeResult _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN int		Tcl_GetAlias _ANSI_ARGS_((Tcl_Interp *interp,
       			    char *slaveCmd, Tcl_Interp **targetInterpPtr,
							 char **targetCmdPtr, int *argcPtr,
				char ***argvPtr));
EXTERN int		Tcl_GetAliasObj _ANSI_ARGS_((Tcl_Interp *interp,
						char *slaveCmd, Tcl_Interp **targetInterpPtr,
                            char **targetCmdPtr, int *objcPtr,
				Tcl_Obj ***objv));
EXTERN ClientData	Tcl_GetAssocData _ANSI_ARGS_((Tcl_Interp *interp,
                            char *name, Tcl_InterpDeleteProc **procPtr));
EXTERN int		Tcl_GetBoolean _ANSI_ARGS_((Tcl_Interp *interp,
				char *string, int *boolPtr));
EXTERN int		Tcl_GetBooleanFromObj _ANSI_ARGS_((
			    Tcl_Interp *interp, Tcl_Obj *objPtr,
			    int *boolPtr));
EXTERN unsigned char *	Tcl_GetByteArrayFromObj _ANSI_ARGS_((
				Tcl_Obj *objPtr, int *lengthPtr));
EXTERN Tcl_Channel	Tcl_GetChannel _ANSI_ARGS_((Tcl_Interp *interp,
					char *chanName, int *modePtr));
EXTERN int		Tcl_GetChannelBufferSize _ANSI_ARGS_((
    			    Tcl_Channel chan));
EXTERN int		Tcl_GetChannelHandle _ANSI_ARGS_((Tcl_Channel chan,
	        	    int direction, ClientData *handlePtr));
EXTERN ClientData	Tcl_GetChannelInstanceData _ANSI_ARGS_((
    			    Tcl_Channel chan));
EXTERN int		Tcl_GetChannelMode _ANSI_ARGS_((Tcl_Channel chan));
EXTERN char *		Tcl_GetChannelName _ANSI_ARGS_((Tcl_Channel chan));
EXTERN int		Tcl_GetChannelOption _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Channel chan, char *optionName,
			    Tcl_DString *dsPtr));
EXTERN Tcl_ChannelType * Tcl_GetChannelType _ANSI_ARGS_((Tcl_Channel chan));
EXTERN int		Tcl_GetCommandInfo _ANSI_ARGS_((Tcl_Interp *interp,
				char *cmdName, Tcl_CmdInfo *infoPtr));
EXTERN char *		Tcl_GetCommandName _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Command command));
EXTERN Tcl_ThreadId	Tcl_GetCurrentThread _ANSI_ARGS_((void));
EXTERN int		Tcl_GetDouble _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, double *doublePtr));
EXTERN int		Tcl_GetDoubleFromObj _ANSI_ARGS_((
			    Tcl_Interp *interp, Tcl_Obj *objPtr,
				double *doublePtr));
EXTERN Tcl_Encoding	Tcl_GetEncoding _ANSI_ARGS_((Tcl_Interp *interp,
			    CONST char *name));
EXTERN char *		Tcl_GetEncodingName _ANSI_ARGS_((
			    Tcl_Encoding encoding));
EXTERN void		Tcl_GetEncodingNames _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN Tcl_Obj *	Tcl_GetEncodingPath _ANSI_ARGS_((void));
EXTERN int		Tcl_GetErrno _ANSI_ARGS_((void));
EXTERN char *		Tcl_GetHostName _ANSI_ARGS_((void));
EXTERN int		Tcl_GetIndexFromObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *objPtr, char **tablePtr, char *msg,
			    int flags, int *indexPtr));
EXTERN int		Tcl_GetIndexFromObjStruct _ANSI_ARGS_((
			    Tcl_Interp *interp, Tcl_Obj *objPtr, 
			    char **tablePtr, int offset, char *msg, int flags,
			    int *indexPtr));
EXTERN int		Tcl_GetInt _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, int *intPtr));
EXTERN int		Tcl_GetInterpPath _ANSI_ARGS_((Tcl_Interp *askInterp,
			    Tcl_Interp *slaveInterp));
EXTERN int		Tcl_GetIntFromObj _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Obj *objPtr, int *intPtr));
EXTERN int		Tcl_GetLongFromObj _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Obj *objPtr, long *longPtr));
EXTERN Tcl_Interp *	Tcl_GetMaster _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN CONST char *	Tcl_GetNameOfExecutable _ANSI_ARGS_((void));
EXTERN Tcl_Obj *	Tcl_GetObjResult _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN Tcl_Obj *	Tcl_GetObjVar2 _ANSI_ARGS_((Tcl_Interp *interp,
			    char *part1, char *part2, int flags));
EXTERN Tcl_ObjType *	Tcl_GetObjType _ANSI_ARGS_((char *typeName));
EXTERN int		Tcl_GetOpenFile _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, int write, int checkUsage,
				ClientData *filePtr));
EXTERN Tcl_PathType	Tcl_GetPathType _ANSI_ARGS_((char *path));
EXTERN int		Tcl_Gets _ANSI_ARGS_((Tcl_Channel chan,
					Tcl_DString *dsPtr));
EXTERN int		Tcl_GetsObj _ANSI_ARGS_((Tcl_Channel chan,
					Tcl_Obj *objPtr));
EXTERN int		Tcl_GetServiceMode _ANSI_ARGS_((void));
EXTERN Tcl_Interp *	Tcl_GetSlave _ANSI_ARGS_((Tcl_Interp *interp,
				char *slaveName));
EXTERN Tcl_Channel	Tcl_GetStdChannel _ANSI_ARGS_((int type));
EXTERN char *		Tcl_GetString _ANSI_ARGS_((Tcl_Obj *objPtr));
EXTERN char *		Tcl_GetStringFromObj _ANSI_ARGS_((Tcl_Obj *objPtr,
			    int *lengthPtr));
EXTERN char *		Tcl_GetStringResult _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN VOID *		Tcl_GetThreadData _ANSI_ARGS_((
			    Tcl_ThreadDataKey *keyPtr, int size));
EXTERN char *		Tcl_GetVar _ANSI_ARGS_((Tcl_Interp *interp,
			    char *varName, int flags));
EXTERN char *		Tcl_GetVar2 _ANSI_ARGS_((Tcl_Interp *interp,
			    char *part1, char *part2, int flags));
EXTERN int		Tcl_GlobalEval _ANSI_ARGS_((Tcl_Interp *interp,
			    char *command));
EXTERN char *		Tcl_HashStats _ANSI_ARGS_((Tcl_HashTable *tablePtr));
EXTERN int		Tcl_HideCommand _ANSI_ARGS_((Tcl_Interp *interp,
		            char *cmdName, char *hiddenCmdToken));
EXTERN int		Tcl_Init _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN void		Tcl_InitHashTable _ANSI_ARGS_((Tcl_HashTable *tablePtr,
			    int keyType));
EXTERN void		Tcl_InitMemory _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN ClientData	Tcl_InitNotifier _ANSI_ARGS_((void));
EXTERN int		Tcl_InputBlocked _ANSI_ARGS_((Tcl_Channel chan));
EXTERN int		Tcl_InputBuffered _ANSI_ARGS_((Tcl_Channel chan));
EXTERN int		Tcl_InterpDeleted _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN int		Tcl_IsSafe _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN void		Tcl_InvalidateStringRep _ANSI_ARGS_((
				Tcl_Obj *objPtr));
EXTERN char *		Tcl_JoinPath _ANSI_ARGS_((int argc, char **argv,
			    Tcl_DString *resultPtr));
EXTERN int		Tcl_LinkVar _ANSI_ARGS_((Tcl_Interp *interp,
				char *varName, char *addr, int type));
EXTERN int		Tcl_ListObjAppendList _ANSI_ARGS_((
			    Tcl_Interp *interp, Tcl_Obj *listPtr, 
			    Tcl_Obj *elemListPtr));
EXTERN int		Tcl_ListObjAppendElement _ANSI_ARGS_((
			    Tcl_Interp *interp, Tcl_Obj *listPtr,
			    Tcl_Obj *objPtr));
EXTERN int		Tcl_ListObjGetElements _ANSI_ARGS_((
				Tcl_Interp *interp, Tcl_Obj *listPtr,
			    int *objcPtr, Tcl_Obj ***objvPtr));
EXTERN int		Tcl_ListObjIndex _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Obj *listPtr, int index,
				Tcl_Obj **objPtrPtr));
EXTERN int		Tcl_ListObjLength _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *listPtr, int *intPtr));
EXTERN int		Tcl_ListObjReplace _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Obj *listPtr, int first, int count,
			    int objc, Tcl_Obj *CONST objv[]));
EXTERN void		Tcl_Main _ANSI_ARGS_((int argc, char **argv,
			    Tcl_AppInitProc *appInitProc));
EXTERN Tcl_Channel	Tcl_MakeFileChannel _ANSI_ARGS_((ClientData handle,
			    int mode));
EXTERN int		Tcl_MakeSafe _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN Tcl_Channel	Tcl_MakeTcpClientChannel _ANSI_ARGS_((
					ClientData tcpSocket));
EXTERN char *		Tcl_Merge _ANSI_ARGS_((int argc, char **argv));
#ifdef TCL_THREADS
EXTERN void		Tcl_MutexLock _ANSI_ARGS_((Tcl_Mutex *mutexPtr));
EXTERN void		Tcl_MutexUnlock _ANSI_ARGS_((Tcl_Mutex *mutexPtr));
EXTERN void		Tcl_ConditionNotify _ANSI_ARGS_((Tcl_Condition *condPtr));
EXTERN void		Tcl_ConditionWait _ANSI_ARGS_((Tcl_Condition *condPtr,
				Tcl_Mutex *mutexPtr, Tcl_Time *timePtr));
#else
#define Tcl_MutexLock(mutexPtr)
#define Tcl_MutexUnlock(mutexPtr)
#define Tcl_ConditionNotify(condPtr)
#define Tcl_ConditionWait(condPtr, mutexPtr, timePtr)
#endif /* TCL_THREADS */
EXTERN Tcl_HashEntry *	Tcl_NextHashEntry _ANSI_ARGS_((
				Tcl_HashSearch *searchPtr));
EXTERN void		Tcl_NotifyChannel _ANSI_ARGS_((Tcl_Channel channel,
				int mask));
EXTERN int		Tcl_NumUtfChars _ANSI_ARGS_((CONST char *src,
				int len));
EXTERN Tcl_Channel	Tcl_OpenCommandChannel _ANSI_ARGS_((
					Tcl_Interp *interp, int argc, char **argv,
			    int flags));
EXTERN Tcl_Channel	Tcl_OpenFileChannel _ANSI_ARGS_((Tcl_Interp *interp,
        		    char *fileName, char *modeString,
                            int permissions));
EXTERN Tcl_Channel	Tcl_OpenTcpClient _ANSI_ARGS_((Tcl_Interp *interp,
			    int port, char *address, char *myaddr,
					int myport, int async));
EXTERN Tcl_Channel	Tcl_OpenTcpServer _ANSI_ARGS_((Tcl_Interp *interp,
		            int port, char *host,
        		    Tcl_TcpAcceptProc *acceptProc,
			    ClientData callbackData));
EXTERN char *		Tcl_ParseVar _ANSI_ARGS_((Tcl_Interp *interp,
				char *string, char **termPtr));
*)
procedure TclPkgProvide(interp: PTclInterp; name,version: PChar); cdecl;
function TclPkgRequire(interp: PTclInterp; name, version: PChar;
								 exact: Integer): PChar; cdecl;
function TclPosixError(interp: PTclInterp): PChar; cdecl;
(*
EXTERN void		Tcl_Preserve _ANSI_ARGS_((ClientData data));
EXTERN void		Tcl_PrintDouble _ANSI_ARGS_((Tcl_Interp *interp,
				double value, char *dst));
EXTERN int		Tcl_PutEnv _ANSI_ARGS_((CONST char *string));
EXTERN void		Tcl_QueueEvent _ANSI_ARGS_((Tcl_Event *evPtr,
				Tcl_QueuePosition position));
EXTERN int		Tcl_Read _ANSI_ARGS_((Tcl_Channel chan,
					char *bufPtr, int toRead));
EXTERN int		Tcl_ReadChars _ANSI_ARGS_((Tcl_Channel channel,
				Tcl_Obj *objPtr, int charsToRead, int appendFlag));
EXTERN void		Tcl_ReapDetachedProcs _ANSI_ARGS_((void));
EXTERN int		Tcl_RecordAndEval _ANSI_ARGS_((Tcl_Interp *interp,
				char *cmd, int flags));
EXTERN int		Tcl_RecordAndEvalObj _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Obj *cmdPtr, int flags));
EXTERN Tcl_RegExp	Tcl_RegExpCompile _ANSI_ARGS_((Tcl_Interp *interp,
				char *string));
EXTERN int		Tcl_RegExpExec _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_RegExp re, CONST char *string,
			    CONST char *start));
EXTERN int		Tcl_RegExpMatch _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, char *pattern));
EXTERN void		Tcl_RegExpRange _ANSI_ARGS_((Tcl_RegExp re,
			    int index, char **startPtr, char **endPtr));
EXTERN void		Tcl_RegisterChannel _ANSI_ARGS_((Tcl_Interp *interp,
					Tcl_Channel chan));
EXTERN void		Tcl_RegisterObjType _ANSI_ARGS_((
			    Tcl_ObjType *typePtr));
EXTERN void		Tcl_Release _ANSI_ARGS_((ClientData clientData));
EXTERN void		Tcl_ResetResult _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN void		Tcl_RestoreResult _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_SavedResult *statePtr));
#define Tcl_Return Tcl_SetResult
EXTERN void		Tcl_SaveResult _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_SavedResult *statePtr));
EXTERN int		Tcl_ScanCountedElement _ANSI_ARGS_((CONST char *string,
				int length, int *flagPtr));
EXTERN int		Tcl_ScanElement _ANSI_ARGS_((CONST char *string,
				int *flagPtr));
EXTERN int		Tcl_Seek _ANSI_ARGS_((Tcl_Channel chan,
        		    int offset, int mode));
EXTERN int		Tcl_ServiceAll _ANSI_ARGS_((void));
EXTERN int		Tcl_ServiceEvent _ANSI_ARGS_((int flags));
EXTERN void		Tcl_SetAssocData _ANSI_ARGS_((Tcl_Interp *interp,
                            char *name, Tcl_InterpDeleteProc *proc,
							 ClientData clientData));
EXTERN void		Tcl_SetBooleanObj _ANSI_ARGS_((Tcl_Obj *objPtr, 
			    int boolValue));
EXTERN unsigned char *	Tcl_SetByteArrayLength _ANSI_ARGS_((Tcl_Obj *objPtr, 
			    int length));
EXTERN void		Tcl_SetByteArrayObj _ANSI_ARGS_((Tcl_Obj *objPtr,
				unsigned char *bytes, int length));
EXTERN void		Tcl_SetChannelBufferSize _ANSI_ARGS_((
			    Tcl_Channel chan, int sz));
EXTERN int		Tcl_SetChannelOption _ANSI_ARGS_((
			    Tcl_Interp *interp, Tcl_Channel chan,
					char *optionName, char *newValue));
EXTERN int		Tcl_SetCommandInfo _ANSI_ARGS_((Tcl_Interp *interp,
			    char *cmdName, Tcl_CmdInfo *infoPtr));
EXTERN void		Tcl_SetDoubleObj _ANSI_ARGS_((Tcl_Obj *objPtr,
			    double doubleValue));
EXTERN void		Tcl_SetErrno _ANSI_ARGS_((int err));
EXTERN void		Tcl_SetErrorCode _ANSI_ARGS_(
					TCL_VARARGS(Tcl_Interp *,arg1));
EXTERN void		Tcl_SetIntObj _ANSI_ARGS_((Tcl_Obj *objPtr,
				int intValue));
EXTERN void		Tcl_SetListObj _ANSI_ARGS_((Tcl_Obj *objPtr, 
				int objc, Tcl_Obj *CONST objv[]));
EXTERN void		Tcl_SetLongObj _ANSI_ARGS_((Tcl_Obj *objPtr, 
			    long longValue));
EXTERN void		Tcl_SetMaxBlockTime _ANSI_ARGS_((Tcl_Time *timePtr));
EXTERN void		Tcl_SetObjErrorCode _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *errorObjPtr));
EXTERN void		Tcl_SetObjLength _ANSI_ARGS_((Tcl_Obj *objPtr,
				int length));
EXTERN void		Tcl_SetObjResult _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *resultObjPtr));
EXTERN Tcl_Obj *	Tcl_SetObjVar2 _ANSI_ARGS_((Tcl_Interp *interp,
			    char *part1, char *part2, Tcl_Obj *newValuePtr,
			    int flags));
EXTERN void		Tcl_SetPanicProc _ANSI_ARGS_((void (*proc)
			    _ANSI_ARGS_(TCL_VARARGS(char *, format))));
EXTERN int		Tcl_SetRecursionLimit _ANSI_ARGS_((Tcl_Interp *interp,
				int depth));
EXTERN void		Tcl_SetResult _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, Tcl_FreeProc *freeProc));
EXTERN int		Tcl_SetServiceMode _ANSI_ARGS_((int mode));
EXTERN void		Tcl_SetStdChannel _ANSI_ARGS_((Tcl_Channel channel,
				int type));
EXTERN void		Tcl_SetStringObj _ANSI_ARGS_((Tcl_Obj *objPtr, 
				char *bytes, int length));
EXTERN void		Tcl_SetTimer _ANSI_ARGS_((Tcl_Time *timePtr));
EXTERN int		Tcl_SetSystemEncoding _ANSI_ARGS_((Tcl_Interp *interp,
			    CONST char *name));
EXTERN char *		Tcl_SetVar _ANSI_ARGS_((Tcl_Interp *interp,
				char *varName, char *newValue, int flags));
EXTERN char *		Tcl_SetVar2 _ANSI_ARGS_((Tcl_Interp *interp,
			    char *part1, char *part2, char *newValue,
			    int flags));
EXTERN char *		Tcl_SignalId _ANSI_ARGS_((int sig));
EXTERN char *		Tcl_SignalMsg _ANSI_ARGS_((int sig));
EXTERN void		Tcl_Sleep _ANSI_ARGS_((int ms));
EXTERN void		Tcl_SourceRCFile _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN int		Tcl_SplitList _ANSI_ARGS_((Tcl_Interp *interp,
			    CONST char *list, int *argcPtr, char ***argvPtr));
EXTERN void		Tcl_SplitPath _ANSI_ARGS_((CONST char *path,
				int *argcPtr, char ***argvPtr));
EXTERN void		Tcl_StaticPackage _ANSI_ARGS_((Tcl_Interp *interp,
			    char *pkgName, Tcl_PackageInitProc *initProc,
				Tcl_PackageInitProc *safeInitProc));
EXTERN int		Tcl_StringMatch _ANSI_ARGS_((CONST char *string,
			    CONST char *pattern));
EXTERN int		Tcl_Tell _ANSI_ARGS_((Tcl_Channel chan));
EXTERN void		Tcl_ThreadAlert _ANSI_ARGS_((Tcl_ThreadId threadId));
EXTERN void		Tcl_ThreadQueueEvent _ANSI_ARGS_((
				Tcl_ThreadId threadId, Tcl_Event* evPtr,
				Tcl_QueuePosition position));
#define Tcl_TildeSubst Tcl_TranslateFileName
EXTERN int		Tcl_TraceVar _ANSI_ARGS_((Tcl_Interp *interp,
				char *varName, int flags, Tcl_VarTraceProc *proc,
				ClientData clientData));
EXTERN int		Tcl_TraceVar2 _ANSI_ARGS_((Tcl_Interp *interp,
			    char *part1, char *part2, int flags,
				Tcl_VarTraceProc *proc, ClientData clientData));
EXTERN char *		Tcl_TranslateFileName _ANSI_ARGS_((Tcl_Interp *interp,
				char *name, Tcl_DString *bufferPtr));
EXTERN int		Tcl_Ungets _ANSI_ARGS_((Tcl_Channel chan, char *str,
				int len, int atHead));
EXTERN Tcl_UniChar	Tcl_UniCharAtIndex _ANSI_ARGS_((CONST char *src,
			    int index));
EXTERN Tcl_UniChar	Tcl_UniCharToLower _ANSI_ARGS_((int ch));
EXTERN Tcl_UniChar	Tcl_UniCharToTitle _ANSI_ARGS_((int ch));
EXTERN Tcl_UniChar	Tcl_UniCharToUpper _ANSI_ARGS_((int ch));
EXTERN int		Tcl_UniCharToUtf _ANSI_ARGS_((int ch,
			    char *buf));
EXTERN void		Tcl_UnlinkVar _ANSI_ARGS_((Tcl_Interp *interp,
			    char *varName));
EXTERN int		Tcl_UnregisterChannel _ANSI_ARGS_((Tcl_Interp *interp,
				Tcl_Channel chan));
EXTERN int		Tcl_UnsetVar _ANSI_ARGS_((Tcl_Interp *interp,
			    char *varName, int flags));
EXTERN int		Tcl_UnsetVar2 _ANSI_ARGS_((Tcl_Interp *interp,
			    char *part1, char *part2, int flags));
EXTERN void		Tcl_UntraceVar _ANSI_ARGS_((Tcl_Interp *interp,
			    char *varName, int flags, Tcl_VarTraceProc *proc,
			    ClientData clientData));
EXTERN void		Tcl_UntraceVar2 _ANSI_ARGS_((Tcl_Interp *interp,
				char *part1, char *part2, int flags,
				Tcl_VarTraceProc *proc, ClientData clientData));
EXTERN void		Tcl_UpdateLinkedVar _ANSI_ARGS_((Tcl_Interp *interp,
				char *varName));
EXTERN int		Tcl_UpVar _ANSI_ARGS_((Tcl_Interp *interp,
				char *frameName, char *varName,
			    char *localName, int flags));
EXTERN int		Tcl_UpVar2 _ANSI_ARGS_((Tcl_Interp *interp,
			    char *frameName, char *part1, char *part2,
			    char *localName, int flags));
EXTERN char *		Tcl_UtfAtIndex _ANSI_ARGS_((CONST char *src,
			    int index));
EXTERN int		Tcl_UtfCharComplete _ANSI_ARGS_((CONST char *src,
				int len));
EXTERN int		Tcl_UtfBackslash _ANSI_ARGS_((CONST char *src,
			    int *readPtr, char *dst));
EXTERN char *		Tcl_UtfFindFirst _ANSI_ARGS_((CONST char *src,
			    int ch));
EXTERN char *		Tcl_UtfFindLast _ANSI_ARGS_((CONST char *src,
			    int ch));
EXTERN int		Tcl_UtfIsLower _ANSI_ARGS_((CONST char *src));
EXTERN int		Tcl_UtfIsUpper _ANSI_ARGS_((CONST char *src));
EXTERN char *		Tcl_UtfNext _ANSI_ARGS_((CONST char *src));
EXTERN char *		Tcl_UtfPrev _ANSI_ARGS_((CONST char *src,
			    CONST char *start));
EXTERN int		Tcl_UtfToExternal _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Encoding encoding, CONST char *src, int srcLen,
			    int flags, Tcl_EncodingState *statePtr, char *dst,
				int dstLen, int *srcReadPtr, int *dstWrotePtr,
			    int *dstCharsPtr));
EXTERN char *		Tcl_UtfToExternalDString _ANSI_ARGS_((
			    Tcl_Encoding encoding, CONST char *src,
				int srcLen, Tcl_DString *dsPtr));
EXTERN int		Tcl_UtfToLower _ANSI_ARGS_((char *src));
EXTERN int		Tcl_UtfToTitle _ANSI_ARGS_((char *src));
EXTERN int		Tcl_UtfToUniChar _ANSI_ARGS_((CONST char *src,
				Tcl_UniChar *chPtr));
EXTERN int		Tcl_UtfToUpper _ANSI_ARGS_((char *src));
EXTERN int		Tcl_VarEval _ANSI_ARGS_(
    			    TCL_VARARGS(Tcl_Interp *,interp));
EXTERN ClientData	Tcl_VarTraceInfo _ANSI_ARGS_((Tcl_Interp *interp,
				char *varName, int flags,
				Tcl_VarTraceProc *procPtr,
				ClientData prevClientData));
EXTERN ClientData	Tcl_VarTraceInfo2 _ANSI_ARGS_((Tcl_Interp *interp,
				char *part1, char *part2, int flags,
				Tcl_VarTraceProc *procPtr,
				ClientData prevClientData));
EXTERN int		Tcl_WaitForEvent _ANSI_ARGS_((Tcl_Time *timePtr));
EXTERN Tcl_Pid		Tcl_WaitPid _ANSI_ARGS_((Tcl_Pid pid, int *statPtr,
				int options));
EXTERN int		Tcl_Write _ANSI_ARGS_((Tcl_Channel chan,
				char *src, int srcLen));
EXTERN int		Tcl_WriteChars _ANSI_ARGS_((Tcl_Channel chan,
				CONST char *src, int srcLen));
EXTERN int		Tcl_WriteObj _ANSI_ARGS_((Tcl_Channel chan,
				Tcl_Obj *objPtr));
*)
procedure TclWrongNumArgs(interp: PTclInterp; objc: Integer; objv: PObjArgs;
								msg: PChar); cdecl;

implementation

{* Pull the actual functions from the DLL.  We use TCL.DLL, not TCL80.DLL, *}
{* to make it easier to upgrade the TCL library on the client PC.			*}
function TclNewBooleanObj; external 'TCL.DLL' name 'Tcl_NewBooleanObj';
function TclNewByteArrayObj; external 'TCL.DLL' name 'Tcl_NewByteArrayObj';
function TclNewDoubleObj; external 'TCL.DLL' name 'Tcl_NewDoubleObj';
function TclNewIntObj; external 'TCL.DLL' name 'Tcl_NewIntObj';
function TclNewListObj; external 'TCL.DLL' name 'Tcl_NewListObj';
function TclNewLongObj; external 'TCL.DLL' name 'Tcl_NewLongObj';
function TclNewObj; external 'TCL.DLL' name 'Tcl_NewObj';
function TclNewStringObj; external 'TCL.DLL' name 'Tcl_NewStringObj';

procedure TclIncrRefCount; external 'TCL.DLL' name 'Tcl_IncrRefCount';
procedure TclDecrRefCount; external 'TCL.DLL' name 'Tcl_DecrRefCount';
function TclIsShared; external 'TCL.DLL' name 'Tcl_IsShared';

procedure TclAddErrorInfo; external 'TCL.DLL' name 'Tcl_AddErrorInfo';
procedure TclAddObjErrorInfo; external 'TCL.DLL' name 'Tcl_AddObjErrorInfo';
procedure TclAllowExceptions; external 'TCL.DLL' name 'Tcl_AllowExceptions';
function TclAppendAllObjTypes; external 'TCL.DLL' name 'Tcl_AppendAllObjTypes';
procedure TclAppendElement; external 'TCL.DLL' name 'Tcl_AppendElement';
procedure TclAppendResult; external 'TCL.DLL' name 'Tcl_AppendResult';
procedure TclAppendObjToObj; external 'TCL.DLL' name 'Tcl_AppendObjToObj';
procedure TclAppendToObj; external 'TCL.DLL' name 'Tcl_AppendToObj';
function TclAppInit; external 'TCL.DLL' name 'Tcl_AppInit';
procedure TclAlertNotifier; external 'TCL.DLL' name 'Tcl_AlertNotifier';
function TclAsyncCreate; external 'TCL.DLL' name 'Tcl_AsyncCreate';
procedure TclAsyncDelete; external 'TCL.DLL' name 'Tcl_AsyncDelete';
function TclAsyncInvoke; external 'TCL.DLL' name 'Tcl_AsyncInvoke';
procedure TclAsnycMark; external 'TCL.DLL' name 'Tcl_AsyncMark';
function TclAsyncReady; external 'TCL.DLL' name 'Tcl_AsyncReady';
procedure TclBackgroundError; external 'TCL.DLL' name 'Tcl_BackgroundError';
function TclBackslash; external 'TCL.DLL' name 'Tcl_Backslash';
function TclBadChannelOption; external 'TCL.DLL' name 'Tcl_BadChannelOption';
procedure TclCallWhenDelete; external 'TCL.DLL' name 'Tcl_CallWhenDelete';
procedure TclCancelIdleCall; external 'TCL.DLL' name 'Tcl_CancelIdleCall';
function TclCreateCommand; external 'TCL.DLL' name 'Tcl_CreateCommand';
procedure TclCreateFileHandler; external 'TCL.DLL' name 'Tcl_CreateFileHandler';
function TclCreateInterp; external 'TCL.DLL' name 'Tcl_CreateInterp';
function TclDeleteCommand; external 'TCL.DLL' name 'Tcl_DeleteCommand';
procedure TclDeleteInterp; external 'TCL.DLL' name 'Tcl_DeleteInterp';
function TclEval; external 'TCL.DLL' name 'Tcl_Eval';
function TclEval2; external 'TCL.DLL' name 'Tcl_Eval2';
function TclEvalFile; external 'TCL.DLL' name 'Tcl_EvalFile';
procedure TclPkgProvide; external 'TCL.DLL' name 'Tcl_PkgProvide';
function TclPkgRequire; external 'TCL.DLL' name 'Tcl_PkgRequire';
function TclPosixError; external 'TCL.DLL' name 'Tcl_PosixError';
procedure TclCreateExitHandler; external 'TCL.DLL' name 'Tcl_CreateExitHandler';
procedure TclWrongNumArgs; external 'TCL.DLL' name 'Tcl_WrongNumArgs';

end.
